#version 330 core

in float height;

in vec3 toCameraVector;
in vec3 normalPass;
in vec2 uvPass;
in vec4 positionRelativeToCam;

out vec4 color;

uniform sampler2D tex;
uniform float changebrightness;

// material
uniform float shineDamper;
uniform float reflectiveness;
uniform float alpha;

uniform float offset;

// cel shading
int celLevels = 3;

// fog
const float density = 0.0012;
const float gradient = 2;

float PI = 3.14159265;

float rand(vec2 n) 
{ 
	return fract(sin(dot(n, vec2(12.9898, 4.1414))) * 43758.5453);
}

float noise(vec2 p)
{
	vec2 ip = floor(p);
	vec2 u = fract(p);
	u = u*u*(3.0-2.0*u);
	
	float res = mix(
		mix(rand(ip),rand(ip+vec2(1.0,0.0)),u.x),
		mix(rand(ip+vec2(0.0,1.0)),rand(ip+vec2(1.0,1.0)),u.x),u.y);
	return res*res;
}

void main()
{
	// calculate normal
	vec4 texcol = texture(tex, uvPass);
	float value = sin((uvPass.x * 200) + offset * 2*PI + noise(uvPass * 200) * 10)/2 + 0.5;
	//texcol = vec4(vec3(value, 0, 0), 1);
	//texcol = vec4(vec3(0.24, 0, 0.58), 1);
	//texcol = vec4(vec3(0.13 + 0.11 * value, 0, 0.31 + 0.27 * value), 1);
	texcol = vec4(vec3(0.1 + 0.04 * value, 0, 0.20 + 0.15 * value), 1);
	
	vec3 unitNormal = normalize(normalPass);
	vec3 unitVectorToCamera = normalize(toCameraVector);

	// calculate diffuse lighting	
	vec3 lightDirection = vec3(0, -1, -1);
	vec3 unitLightVector = -normalize(lightDirection.xyz);
	float nDotl = dot(unitNormal, unitLightVector);	
	float brightness = max(nDotl, 0);
	brightness = min(1, brightness+changebrightness);
	
	// cel shading
	float level = floor(brightness * celLevels);
	brightness = level / celLevels;
	
	// calculate specular lighting
	float damperFactor;
	if (nDotl > 0) 
	{
		vec3 reflectedLightDirection = reflect(lightDirection, unitNormal);
		float specularFactor = dot(reflectedLightDirection, unitVectorToCamera);
		specularFactor = max(specularFactor, 0.0);
		damperFactor = pow(specularFactor, shineDamper);

		// cel shading
		level = floor(damperFactor * celLevels);
		damperFactor = level / celLevels;
	}
	else
	{
		damperFactor = 0;
	}
	
	vec3 lightColor = vec3(1, 1, 1);
	
	vec4 attdiffColor = vec4((brightness) * lightColor, 1) * vec4(texcol.xyz, 1);
	vec4 specularColor = vec4(damperFactor * reflectiveness * lightColor, 1);
	
	color = attdiffColor + specularColor;
	
	float distance = length(positionRelativeToCam.xyz) * 200;
	float visibility = exp(-pow((distance*density), gradient));
	visibility = clamp(visibility, 0.0, 1.0);
	
	color = mix(vec4(vec4(0.05, 0.01, 0.10, 1)), texcol, visibility);
}