#version 330 core

// Input vertex data, different for all executions of this shader.
layout(location = 0) in vec3 vertexPosition;
layout(location = 1) in vec2 vertexUV;
layout(location = 2) in vec3 normal;

out float height;

out vec3 normalPass;
out vec2 uvPass;
out vec3 toCameraVector;
out vec4 positionRelativeToCam;

uniform mat4 viewMatrix;
uniform mat4 viewMatrixinv;
uniform mat4 transformationMatrix;
uniform mat4 projectionMatrix;

// clipping -------------------------------------
uniform float clippingMode = 0;

void setClippingPlane(float height)
{
	if(clippingMode == 0)
	{
		gl_ClipDistance[0] = 1;
	}
	else
	{
		gl_ClipDistance[0] = height * sign(clippingMode) + 0.05;
	}
}
// ----------------------------------------------

void main()
{
	vec4 worldPosition = transformationMatrix * vec4(vertexPosition.x, vertexPosition.y, vertexPosition.z, 1.0);
	
	positionRelativeToCam = viewMatrix * worldPosition;
	
	setClippingPlane(worldPosition.y);
	
	normalPass = (transformationMatrix * vec4(normal, 0.0)).xyz;
	uvPass = vertexUV;

	// Output position of the vertex, in clip space : MVP * position
	gl_Position = projectionMatrix * positionRelativeToCam;
	
	toCameraVector = (viewMatrixinv * vec4(0.0,0.0,0.0,1.0)).xyz - worldPosition.xyz;
}

