#version 330 core

in float height;

in vec3 toCameraVector;
in vec3 toCameraVectorNormal;
in vec3 normalPass;
in vec2 uvPass;
in mat3 toTangentSpace;

out vec4 color;

uniform sampler2D tex;
uniform float changebrightness;
uniform sampler2D lightMap;
uniform sampler2D normalMap;
uniform bool lightMapActivated;
uniform bool normalMapActivated;

// material
uniform float shineDamper;
uniform float reflectiveness;
uniform float alpha;

// cel shading
int celLevels = 3;
uniform bool celShade;

// lightmap intnsity
float lightmapIntnsity = 2;

void main()
{
	// calculate normal
	vec4 texcol = texture(tex, uvPass);
	vec3 normalMapValue;
	vec3 unitNormal;
	vec3 unitVectorToCamera;
	

	// calculate diffuse lighting	
	vec3 lightDirection = vec3(0, -0.5, 1);
	vec3 unitLightVector;

	if(normalMapActivated)
	{
		normalMapValue = texture(normalMap,  vec2(uvPass.x, uvPass.y), -1).rgb * 2.0-1.0;
		unitNormal = normalize(normalMapValue);
		unitVectorToCamera = normalize(toCameraVectorNormal);
		unitLightVector = toTangentSpace * -normalize(lightDirection.xyz);
	}
	else
	{
		unitNormal = normalize(normalPass);
		unitVectorToCamera = normalize(toCameraVector);
		unitLightVector = -normalize(lightDirection.xyz);
	}

	float nDotl = dot(unitNormal, unitLightVector);	
	float brightness = max(nDotl, 0);
	
	if(lightMapActivated)
	{
		brightness = min(1, brightness + texture(lightMap, uvPass).x * lightmapIntnsity);
	}

	// cel shading
	float level;
	if(celShade)
	{
		level = floor(brightness * celLevels);
		brightness = level / celLevels;
	}
	
	brightness = min(1, brightness+changebrightness);
	
	// calculate specular lighting
	float damperFactor;
	if (nDotl > 0) 
	{
		vec3 reflectedLightDirection = reflect(lightDirection, unitNormal);
		float specularFactor = dot(reflectedLightDirection, unitVectorToCamera);
		specularFactor = max(specularFactor, 0.0);
		damperFactor = pow(specularFactor, shineDamper);

		// cel shading
		if (celShade)
		{
			level = floor(damperFactor * celLevels);
			damperFactor = level / celLevels;
		}
	}
	else
	{
		damperFactor = 0;
	}
	
	vec3 lightColor = vec3(1, 1, 1);
	
	vec4 attdiffColor = vec4((brightness) * lightColor, 1) * vec4(texcol.xyz, 1);
	vec4 specularColor = vec4(damperFactor * reflectiveness * lightColor, 1);

	color = attdiffColor + specularColor;
	color.a = alpha;
}