#version 330 core

// Input vertex data, different for all executions of this shader.
layout(location = 0) in vec3 vertexPosition;
layout(location = 1) in vec2 vertexUV;
layout(location = 2) in vec3 normal;
layout(location = 3) in vec3 tangent;

out float height;

out vec3 normalPass;
out vec4 clipSpace;
out vec2 uvPass;
out vec3 toCameraVector;
out vec3 toCameraVectorNormal;
out mat3 toTangentSpace;

uniform mat4 viewMatrix;
uniform mat4 viewMatrixinv;
uniform mat4 transformationMatrix;
uniform mat4 projectionMatrix;

// clipping -------------------------------------
uniform float clippingMode = 0;

void setClippingPlane(float height)
{
	if(clippingMode == 0)
	{
		gl_ClipDistance[0] = 1;
	}
	else
	{
		gl_ClipDistance[0] = height * sign(clippingMode) + 0.05;
	}
}
// ----------------------------------------------

void main()
{
	vec4 worldPosition = transformationMatrix * vec4(vertexPosition.x, vertexPosition.y, vertexPosition.z, 1.0);
	
	setClippingPlane(worldPosition.y);
	mat4 modelViewMatrix = viewMatrix * transformationMatrix;
	vec4 positionRelativeToCam = modelViewMatrix * vec4(vertexPosition,1.0);
	
	normalPass = (transformationMatrix * vec4(normal, 0.0)).xyz;
	uvPass = vertexUV;
	clipSpace = projectionMatrix * viewMatrix * worldPosition;
	//neu
	gl_Position = clipSpace;//projectionMatrix * viewMatrix * worldPosition;
	
	vec3 surfaceNormal = (transformationMatrix * vec4(normal, 0.0)).xyz;

	vec3 norm = normalize(surfaceNormal);
	vec3 tang = normalize((modelViewMatrix * vec4(tangent, 0.0)).xyz);
	vec3 bitang = normalize(cross(norm,tang));

	toTangentSpace = mat3(
		tang.x, bitang.x, norm.x,
		tang.y, bitang.y, norm.y,
		tang.z, bitang.z, norm.z
	);

	//toCameraVectorNormal = toTangentSpace * (-positionRelativeToCam.xyz);
	toCameraVectorNormal = toTangentSpace * (viewMatrixinv * vec4(0.0,0.0,0.0,1.0)).xyz - worldPosition.xyz;
	// Output position of the vertex, in clip space : MVP * position
	toCameraVector = (viewMatrixinv * vec4(0.0,0.0,0.0,1.0)).xyz - worldPosition.xyz;
	
}

