#version 330 core

// Input vertex data, different for all executions of this shader.
layout(location = 0) in vec3 vertexPosition;
layout(location = 1) in vec2 vertexUV;
layout(location = 2) in vec3 normal;

out float height;

out vec3 normalPass;
out vec2 uvPass;

uniform mat4 viewMatrix;
uniform mat4 projectionMatrix;
uniform mat4 transformationMatrix;
uniform mat4 frustumMatrix;

// clipping -------------------------------------
uniform float clippingMode = 0;

void setClippingPlane(float height)
{
	if(clippingMode == 0)
	{
		gl_ClipDistance[0] = 1;
	}
	else
	{
		gl_ClipDistance[0] = height * sign(clippingMode);
	}
}
// ----------------------------------------------

void main()
{
	vec4 worldPosition = transformationMatrix * inverse(frustumMatrix) * vec4(vertexPosition.x, vertexPosition.y, vertexPosition.z, 1.0);
	
	gl_ClipDistance[0] = worldPosition.y;
	
	uvPass = vertexUV;
	normalPass = normal;

	gl_Position = projectionMatrix * viewMatrix * worldPosition;
}

