#version 330 core

in float height;

in vec3 toCameraVector;
in vec3 normalPass;
in vec2 uvPass;

out vec4 color;

uniform sampler2D tex;
uniform sampler2D mainscene;
uniform sampler2D mainsceneDepth;
uniform sampler2D hud;
uniform sampler2D bloom;
uniform float changebrightness;

uniform float screenWidth;
uniform float screenHeight;

uniform bool activateBloom;
uniform bool activateEdgeD;

// damage value
uniform float damageValue;

// material
float shineDamper = 1;
float reflectiveness = 0.1;

// edge detection
//float edgePrecision = 0.25;
//float edgeStrenght = 0.5;
float edgePrecision = 1;
float edgeStrenght = 2;

vec4 sobelVertical(sampler2D tx, vec2 uvCoord)
{
	float x = edgePrecision/screenWidth;
	float y = edgePrecision/screenHeight;

	vec4 horizEdge = vec4( 0.0 );
	horizEdge -= texture2D( tx, vec2( uvCoord.x - x, uvCoord.y - y ) ) * 1.0;
	horizEdge -= texture2D( tx, vec2( uvCoord.x - x, uvCoord.y     ) ) * 2.0;
	horizEdge -= texture2D( tx, vec2( uvCoord.x - x, uvCoord.y + y ) ) * 1.0;
	horizEdge += texture2D( tx, vec2( uvCoord.x + x, uvCoord.y - y ) ) * 1.0;
	horizEdge += texture2D( tx, vec2( uvCoord.x + x, uvCoord.y     ) ) * 2.0;
	horizEdge += texture2D( tx, vec2( uvCoord.x + x, uvCoord.y + y ) ) * 1.0;
	vec4 vertEdge = vec4( 0.0 );
	vertEdge -= texture2D( tx, vec2( uvCoord.x - x, uvCoord.y - y ) ) * 1.0;
	vertEdge -= texture2D( tx, vec2( uvCoord.x    , uvCoord.y - y ) ) * 2.0;
	vertEdge -= texture2D( tx, vec2( uvCoord.x + x, uvCoord.y - y ) ) * 1.0;
	vertEdge += texture2D( tx, vec2( uvCoord.x - x, uvCoord.y + y ) ) * 1.0;
	vertEdge += texture2D( tx, vec2( uvCoord.x    , uvCoord.y + y ) ) * 2.0;
	vertEdge += texture2D( tx, vec2( uvCoord.x + x, uvCoord.y + y ) ) * 1.0;
	return sqrt((horizEdge.rgba * horizEdge.rgba) + (vertEdge.rgba * vertEdge.rgba));
}

vec4 mergeEdgeValue(vec4 color, vec4 edgeVal, float edgeSt)
{
	float newEdgeVal = min(edgeVal.x + edgeVal.y + edgeVal.z, 1) * edgeSt;

	return vec4(max(color.xyz - newEdgeVal, 0), color.a);
}

vec4 mergeHUD(vec4 color, vec4 hudVal)
{
	float dimod = 16;

	float xdistance = abs(0.5 - uvPass.x);
	float ydistance = abs(0.5 - uvPass.y);
	float distance = sqrt(xdistance*xdistance + ydistance*ydistance);
	
	float value = min(1, max(damageValue*dimod*pow(distance, 4), 0));
	
	return vec4(max(color.xyz + hudVal.xyz * value, 0), color.a);
}

vec4 mergeBloom(vec4 color, vec4 bloomVal)
{
	return vec4(min(max(color.xyz + bloomVal.xyz, 0), 1), color.a);
}

void main()
{
	vec2 uvMain = vec2(uvPass.x, uvPass.y);
	
	vec4 mainCol = texture(mainscene, uvMain);
	vec4 edgeCol = sobelVertical(mainsceneDepth, uvMain);
	vec4 hudCol = texture(hud, uvMain);
	vec4 bloomCol = texture(bloom, uvMain);
	
	color = mainCol;

	if (activateEdgeD)
	{
		color = mergeEdgeValue(color, edgeCol, edgeStrenght);
	}
	color = mergeHUD(color, hudCol);
	if (activateBloom)
	{
		color = mergeBloom(color, bloomCol);
	}
}