#version 330 core

in float height;

in vec3 toCameraVector;
in vec3 normalPass;
in vec2 uvPass;

out vec4 color;

uniform sampler2D tex;
uniform float changebrightness;
uniform sampler2D normalMap;

// material
float shineDamper = 1;
float reflectiveness = 0.1;

void main()
{	
		// calculate normal
	vec4 texcol = texture(tex, uvPass);
	vec3 unitNormal = normalize(normalPass);
	vec3 unitVectorToCamera = normalize(toCameraVector);
	
	// lighting
	vec4 normalMapColor = texture(normalMap, distortedTexCoords);
	vec3 normal = vec3(normalMapColor.r * 2.0 - 1.0, normalMapColor.b * 3.0, normalMapColor.g * 2.0 - 1.0);
	normal = normalize(normal);
	
	// calculate diffuse lighting	
	vec3 lightDirection = vec3(0, -1, -1);
	vec3 unitLightVector = -normalize(lightDirection.xyz);
	float nDotl = dot(unitNormal, unitLightVector);	
	float brightness = max(nDotl, 0);
	brightness = min(1, brightness+changebrightness);
	
	// calculate specular lighting
	float damperFactor;
	if (nDotl > 0) 
	{
		vec3 reflectedLightDirection = reflect(lightDirection, unitNormal);
		float specularFactor = dot(reflectedLightDirection, unitVectorToCamera);
		specularFactor = max(specularFactor, 0.0);
		damperFactor = pow(specularFactor, shineDamper);
	}
	else
	{
		damperFactor = 0;
	}
	
	vec3 lightColor = vec3(1, 1, 1);
	
	vec4 attdiffColor = vec4((brightness) * lightColor, 1) * vec4(texcol.xyz, 1);
	vec4 specularColor = vec4(damperFactor * reflectiveness * lightColor, 1);
	
	color = attdiffColor + specularColor;

	
}
