#version 330 core

in float height;

in vec3 toCameraVector;
in vec3 normalPass;
in vec2 uvPass;

out vec4 color;

uniform sampler2D tex;
uniform float changebrightness;

// material
uniform float shineDamper;
uniform float reflectiveness;

// cel shading
int celLevels = 3;
uniform bool celShade;

// lightmap intnsity
float lightmapIntnsity = 2;

void main()
{
	// calculate normal
	vec4 texcol = texture(tex, uvPass);
	vec3 unitNormal = normalize(normalPass);
	vec3 unitVectorToCamera = normalize(toCameraVector);

	// calculate diffuse lighting	
	vec3 lightDirection = vec3(0, -0.5, 1);
	vec3 unitLightVector = -normalize(lightDirection.xyz);
	float nDotl = dot(unitNormal, unitLightVector);	
	float brightness = max(nDotl, 0);
	
	// cel shading
	float level;
	if (celShade)
	{
		level = floor(brightness * celLevels);
		brightness = level / celLevels;
	}
	
	brightness = min(1, brightness+changebrightness);
	
	// calculate specular lighting
	float damperFactor;
	if (nDotl > 0) 
	{
		vec3 reflectedLightDirection = reflect(lightDirection, unitNormal);
		float specularFactor = dot(reflectedLightDirection, unitVectorToCamera);
		specularFactor = max(specularFactor, 0.0);
		damperFactor = pow(specularFactor, shineDamper);

		// cel shading
		if (celShade)
		{
			level = floor(damperFactor * celLevels);
			damperFactor = level / celLevels;
		}
	}
	else
	{
		damperFactor = 0;
	}
	
	vec3 lightColor = vec3(1, 1, 1);
	
	vec4 attdiffColor = vec4((brightness) * lightColor, 1) * vec4(texcol.xyz, 1);
	vec4 specularColor = vec4(damperFactor * reflectiveness * lightColor, 1);
	
	color = attdiffColor + specularColor;
}