#version 330 core

// Input vertex data, different for all executions of this shader.
layout(location = 0) in vec3 vertexPosition;
layout(location = 1) in vec2 vertexUV;
layout(location = 2) in vec3 normal;

out float height;

out vec3 normalPass;
out vec2 uvPass;
out vec3 toCameraVector;

uniform mat4 viewMatrix;
uniform mat4 viewMatrixinv;
uniform mat4 transformationMatrix;
uniform mat4 projectionMatrix;

uniform float offset;

// clipping -------------------------------------
uniform float clippingMode = 0;

void setClippingPlane(float height)
{
	if(clippingMode == 0)
	{
		gl_ClipDistance[0] = 1;
	}
	else
	{
		gl_ClipDistance[0] = height * sign(clippingMode) + 0.05;
	}
}
// ----------------------------------------------

float PI = 3.14159265;

float rand(vec2 n) 
{ 
	return fract(sin(dot(n, vec2(12.9898, 4.1414))) * 43758.5453);
}

float noise(vec2 p)
{
	vec2 ip = floor(p);
	vec2 u = fract(p);
	u = u*u*(3.0-2.0*u);
	
	float res = mix(
		mix(rand(ip),rand(ip+vec2(1.0,0.0)),u.x),
		mix(rand(ip+vec2(0.0,1.0)),rand(ip+vec2(1.0,1.0)),u.x),u.y);
	return res*res;
}

void main()
{
	float value = sin((vertexUV.x * 150) + offset * 2*PI + noise(vertexUV * 75) * 10)/2 + 0.5;
	value = (sin((vertexUV.x * 40) + offset * 2*PI + noise(vertexUV * 20) * 10)/2 + 0.5) * 0.1;

	vec4 worldPosition = transformationMatrix * vec4(vertexPosition.x + normal.x * value, vertexPosition.y + normal.y * value, vertexPosition.z + normal.z * value, 1.0);
	
	setClippingPlane(worldPosition.y);
	
	normalPass = (transformationMatrix * vec4(normal, 0.0)).xyz;
	uvPass = vertexUV;

	// Output position of the vertex, in clip space : MVP * position
	gl_Position = projectionMatrix * viewMatrix * worldPosition;
	
	toCameraVector = (viewMatrixinv * vec4(0.0,0.0,0.0,1.0)).xyz - worldPosition.xyz;
}

