#version 330 core

in float height;

in vec3 toCameraVector;
in vec3 normalPass;
in vec2 uvPass;

out vec4 color;

uniform float changebrightness;

// material
uniform float shineDamper;
uniform float reflectiveness;
uniform float translucency;
uniform float offset;

// cel shading
int celLevels = 3;
uniform bool celShade;

float PI = 3.14159265;

float rand(vec2 n) 
{ 
	return fract(sin(dot(n, vec2(12.9898, 4.1414))) * 43758.5453);
}

float noise(vec2 p)
{
	vec2 ip = floor(p);
	vec2 u = fract(p);
	u = u*u*(3.0-2.0*u);
	
	float res = mix(
		mix(rand(ip),rand(ip+vec2(1.0,0.0)),u.x),
		mix(rand(ip+vec2(0.0,1.0)),rand(ip+vec2(1.0,1.0)),u.x),u.y);
	return res*res;
}

void main()
{
	float value = sin((uvPass.x * 150) + offset * 2*PI + noise(uvPass * 75) * 10)/2 + 0.5;
	value = sin((uvPass.x * 40) + offset * 2*PI + noise(uvPass * 20) * 10)/2 + 0.5;

	// calculate normal
	vec4 texcol = vec4(0.23f, 0.35 + value, 0.098f, 1.0f);
	vec3 unitNormal = normalize(normalPass);
	vec3 unitVectorToCamera = normalize(toCameraVector);

	// calculate diffuse lighting
	vec3 lightDirection = vec3(0, -0.5, 1);
	vec3 unitLightVector = -normalize(lightDirection.xyz);
	float nDotl = dot(unitNormal, unitLightVector);	
	float brightness = max(nDotl, 0);
	
	// cel shading
	float level;
	if(celShade)
	{
		level = floor(brightness * celLevels);
		brightness = level / celLevels;
	}
	
	brightness = min(1, brightness+changebrightness);
	
	// calculate specular lighting
	float damperFactor;
	if (nDotl > 0) 
	{
		vec3 reflectedLightDirection = reflect(lightDirection, unitNormal);
		float specularFactor = dot(reflectedLightDirection, unitVectorToCamera);
		specularFactor = max(specularFactor, 0.0);
		damperFactor = pow(specularFactor, shineDamper);

		// cel shading
		if(celShade)
		{
			level = floor(damperFactor * celLevels);
			damperFactor = level / celLevels;
		}
	}
	else
	{
		damperFactor = 0;
	}
	
	vec3 lightColor = vec3(1, 1, 1);
	
	vec4 attdiffColor = vec4((brightness) * lightColor, 1) * vec4(texcol.xyz, 1);
	vec4 specularColor = vec4(damperFactor * reflectiveness * lightColor, 1);
	
	color = attdiffColor + specularColor;
	color.a = translucency;
}