#version 330 core

in float height;

in vec3 toCameraVector;
in vec3 normalPass;
in vec2 uvPass;
in vec4 positionRelativeToCam;

out vec4 color;

uniform sampler2D tex;
uniform float changebrightness;

// material
uniform float shineDamper;
uniform float reflectiveness;
uniform float alpha;

uniform float offset;

// cel shading
int celLevels = 3;

// fog
const float density = 0.0012;
const float gradient = 2;

float PI = 3.14159265;

float rand(vec2 n) 
{ 
	return fract(sin(dot(n, vec2(12.9898, 4.1414))) * 43758.5453);
}

float noise(vec2 p)
{
	vec2 ip = floor(p);
	vec2 u = fract(p);
	u = u*u*(3.0-2.0*u);
	
	float res = mix(
		mix(rand(ip),rand(ip+vec2(1.0,0.0)),u.x),
		mix(rand(ip+vec2(0.0,1.0)),rand(ip+vec2(1.0,1.0)),u.x),u.y);
	return res*res;
}

void main()
{
	// calculate normal
	vec4 texcol = texture(tex, uvPass);
	float value = sin((uvPass.x * 100) + offset * 2*PI + noise(uvPass * 40) * 10)/2 + 0.5;
	
	texcol = mix(vec4(0.8, 0.8, 0.9, 1), vec4(1, 1, 1, 1), value);
	/*if(value < 0.4)
	{
		texcol = mix(texcol, vec4(1, 1, 1, 1), 0.5);
	}*/
	//texcol = vec4(vec3(0.9 + 0.1 * value, 0.3 + 0.6 * value, 0), 1);
	
	float distance = length(positionRelativeToCam.xyz) * 200;
	float visibility = exp(-pow((distance*density), gradient));
	visibility = clamp(visibility, 0.0, 1.0);
	
	color = mix(vec4(vec4(0.05, 0.01, 0.10, 1)), texcol, visibility);
}